# Copyright (c) 2011 gocept gmbh & co. kg
# See also LICENSE.txt

import StringIO
import datetime
import gocept.support.staff
import os.path
import unittest


class TestStaff(unittest.TestCase):

    def test_load_format1(self):
        staff = gocept.support.staff.Staff(
            os.path.join(os.path.dirname(__file__), 'data', 'staffv1.cfg'))
        # Nobody is assigned in week 1
        self.assertEquals([], staff.assignments(datetime.date(2011, 1, 1)))
        self.assertEquals([], staff.assignments(datetime.date(2011, 1, 9)))
        # Alice is primary, bob is secondary in week 2
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 10)))
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 14)))
        # Except on the weekend, only bob is on duty
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 15)))
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 16)))

    def test_load_format2(self):
        staff = gocept.support.staff.Staff(
            os.path.join(os.path.dirname(__file__), 'data', 'staffv2.cfg'))
        # Nobody is assigned in week 1
        self.assertEquals([], staff.assignments(datetime.date(2011, 1, 1)))
        self.assertEquals([], staff.assignments(datetime.date(2011, 1, 9)))
        # Alice is primary, bob is secondary in first week of February
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 1)))
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 4)))
        # Except on the weekend, only bob is on duty
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 5)))
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 6)))

    def test_load_format12_mixed(self):
        # This is basically the data of the previous 2 tests mixed together.
        staff = gocept.support.staff.Staff(
            os.path.join(os.path.dirname(__file__), 'data', 'staffv12mixed.cfg'))
        # Nobody is assigned in week 1
        self.assertEquals([], staff.assignments(datetime.date(2011, 1, 1)))
        self.assertEquals([], staff.assignments(datetime.date(2011, 1, 9)))
        # Alice is primary, bob is secondary in week 2
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 10)))
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 14)))
        # Except on the weekend, only bob is on duty
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 15)))
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 1, 16)))
        # Alice is primary, bob is secondary in first week of February
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 1)))
        self.assertEquals([staff.contacts['alice'], staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 4)))
        # Except on the weekend, only bob is on duty
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 5)))
        self.assertEquals([staff.contacts['bob']],
                          staff.assignments(datetime.date(2011, 2, 6)))

    def test_load_format12_mixed_with_duplicate_is_not_allowed(self):
        # It is invalid to specify the same date twice when mixing formats.
        staff = gocept.support.staff.Staff(
            os.path.join(os.path.dirname(__file__), 'data',
                         'staffv12mixed-duplicate.cfg'))
        self.assertRaises(RuntimeError,
                          staff.assignments, datetime.date(2011, 1, 10))


    def test_month_generator(self):
        out = StringIO.StringIO()
        gocept.support.staff.generate_month('2010-01', out)
        self.assertEquals("""\
[2010-1]
1 = ...
2 = ...
3 = ...

# Monday
4 = ...
5 = ...
6 = ...
7 = ...
8 = ...
9 = ...
10 = ...

# Monday
11 = ...
12 = ...
13 = ...
14 = ...
15 = ...
16 = ...
17 = ...

# Monday
18 = ...
19 = ...
20 = ...
21 = ...
22 = ...
23 = ...
24 = ...

# Monday
25 = ...
26 = ...
27 = ...
28 = ...
29 = ...
30 = ...
31 = ...
""", out.getvalue())
